@@ -368,6 +368,11 @@ def iter_resource_imports(ast) -> Iterator[NodeInfo]:
         yield NodeInfo(tuple(stack), node)
 
 
+def iter_variable_imports(ast) -> Iterator[NodeInfo]:
+    for stack, node in _iter_nodes_filtered(ast, accept_class="VariablesImport"):
+        yield NodeInfo(tuple(stack), node)
+
+
 def iter_keywords(ast) -> Iterator[NodeInfo]:
     for stack, node in _iter_nodes_filtered(ast, accept_class="Keyword"):
         yield NodeInfo(tuple(stack), node)
@@ -521,6 +526,22 @@ def get_resource_import_name_token(ast, token):
     return None
 
 
+def get_variables_import_name_token(ast, token):
+    """
+    If the given ast node is a variables import and the token is its name, return
+    the name token, otherwise, return None.
+    """
+    from robocorp_ls_core.basic import isinstance_name
+
+    if (
+        token.type == token.NAME
+        and isinstance_name(ast, "VariablesImport")
+        and ast.name == token.value  # I.e.: match the name, not the alias.
+    ):
+        return token
+    return None
+
+
 def _copy_of_node_replacing_token(node, token, token_type):
     """
     Workaround to create a new version of the same node but with the first
