@@ -294,18 +294,20 @@ def _obtain_import_location_info(completion_context) -> _ImportLocationInfo:
         if ast_utils.is_library_node_info(node_info):
             import_location_info.library_node_info = node_info
 
-            library_doc = libspec_manager.get_library_info(
-                completion_context.token_value_resolving_variables(node_info.node.name),
-                create=True,
-                current_doc_uri=completion_context.doc.uri,
-            )
-            if library_doc is not None:
-                if library_doc.source:
-                    import_location_info.imported_libraries.add(library_doc.source)
-                else:
-                    import_location_info.imported_libraries.add(library_doc.name)
+            library_name = node_info.node.name
+            if library_name:
+                library_doc = libspec_manager.get_library_info(
+                    completion_context.token_value_resolving_variables(library_name),
+                    create=True,
+                    current_doc_uri=completion_context.doc.uri,
+                )
+                if library_doc is not None:
+                    if library_doc.source:
+                        import_location_info.imported_libraries.add(library_doc.source)
+                    else:
+                        import_location_info.imported_libraries.add(library_doc.name)
 
-        elif ast_utils.is_resource_node_info(node_info):
+        elif ast_utils.is_resource_node_info(node_info) and node_info.node.name:
             import_location_info.resource_node_info = node_info
             import_location_info.imported_resources.add(node_info.node.name)
 
