@@ -300,7 +300,8 @@ class CompletionContext(object):
         ast = self.get_ast()
         ret = []
         for library_import in ast_utils.iter_library_imports(ast):
-            ret.append(library_import.node)
+            if library_import.node.name:
+                ret.append(library_import.node)
         return tuple(ret)
 
     @instance_cache
@@ -310,7 +311,19 @@ class CompletionContext(object):
         ast = self.get_ast()
         ret = []
         for resource in ast_utils.iter_resource_imports(ast):
-            ret.append(resource.node)
+            if resource.node.name:
+                ret.append(resource.node)
+        return tuple(ret)
+
+    @instance_cache
+    def get_variable_imports(self):
+        from robotframework_ls.impl import ast_utils
+
+        ast = self.get_ast()
+        ret = []
+        for resource in ast_utils.iter_variable_imports(ast):
+            if resource.node.name:
+                ret.append(resource.node)
         return tuple(ret)
 
     def token_value_resolving_variables(self, token):
@@ -399,6 +412,9 @@ class CompletionContext(object):
 
         return None
 
+    def get_variable_import_as_doc(self, variables_import) -> Optional[IRobotDocument]:
+        return self.get_resource_import_as_doc(variables_import)
+
     @instance_cache
     def get_resource_imports_as_docs(self) -> Tuple[IRobotDocument, ...]:
         ret: List[IRobotDocument] = []
@@ -412,6 +428,18 @@ class CompletionContext(object):
 
         return tuple(ret)
 
+    @instance_cache
+    def get_variable_imports_as_docs(self) -> Tuple[IRobotDocument, ...]:
+        ret: List[IRobotDocument] = []
+
+        variable_imports = self.get_variable_imports()
+        for variable_import in variable_imports:
+            variable_doc = self.get_variable_import_as_doc(variable_import)
+            if variable_doc is not None:
+                ret.append(variable_doc)
+
+        return tuple(ret)
+
     def _resolve_builtin(self, var_name, value_if_not_found, log_info):
         from robotframework_ls.impl.robot_constants import BUILTIN_VARIABLES_RESOLVED
 
