@@ -7,6 +7,7 @@ from typing import Optional, Dict
 from robocorp_ls_core.protocols import Sentinel
 from robotframework_ls.impl.protocols import ILibraryDoc
 import itertools
+from robocorp_ls_core.watchdog_wrapper import IFSObserver
 
 log = get_logger(__name__)
 
@@ -262,6 +263,7 @@ class _FolderInfo(object):
                     [PathInfo(folder_path, recursive=self.recursive)],
                     notifier.on_change,
                     (self._on_change_spec,),
+                    extensions=(".py", ".libspec"),
                 )
 
     def _on_change_spec(self, spec_file):
@@ -350,7 +352,7 @@ class LibspecManager(object):
     @classmethod
     def get_robot_version(cls):
         try:
-            import robot
+            import robot  # noqa
 
             v = str(robot.get_version())
         except:
@@ -400,7 +402,12 @@ class LibspecManager(object):
     # a libspec for a target filename.
     INTERNAL_VERSION = "v2"
 
-    def __init__(self, builtin_libspec_dir=None, user_libspec_dir=None):
+    def __init__(
+        self,
+        builtin_libspec_dir=None,
+        user_libspec_dir=None,
+        observer: Optional[IFSObserver] = None,
+    ):
         """
         :param __internal_libspec_dir__:
             Only to be used in tests (to regenerate the builtins)!
@@ -411,33 +418,11 @@ class LibspecManager(object):
 
         self._main_thread = threading.current_thread()
 
-        watch_impl = os.environ.get("ROBOTFRAMEWORK_LS_WATCH_IMPL", "auto")
-        if watch_impl not in ("watchdog", "fsnotify", "auto"):
-            log.info(
-                f"ROBOTFRAMEWORK_LS_WATCH_IMPL should be 'auto', 'watchdog' or 'fsnotify'. Found: {watch_impl} (falling back to auto)"
-            )
-            watch_impl = "auto"
-
-        if watch_impl == "auto":
-            # In auto mode we use watchdog for windows and fsnotify (polling)
-            # for Linux and Mac. The reason for that is that on Linux and Mac
-            # if big folders are watched the system may complain due to the
-            # lack of resources, which may prevent the extension from working
-            # properly.
-            #
-            # If users want to opt-in, they can change to watchdog (and
-            # ideally install it to their env to get native extensions).
-            if sys.platform == "win32":
-                watch_impl = "watchdog"
-            else:
-                watch_impl = "fsnotify"
-
-        self._observer = watchdog_wrapper.create_observer(
-            watch_impl, (".py", ".libspec")
-        )
+        assert observer
+        self._fs_observer = observer
 
         self._file_changes_notifier = watchdog_wrapper.create_notifier(
-            self._on_file_changed, timeout=0.5
+            self._on_file_changed, timeout=0.5, extensions=(".libspec", ".py")
         )
 
         self._libspec_dir = self.get_internal_libspec_dir()
@@ -464,20 +449,22 @@ class LibspecManager(object):
             pass
 
         # Spec info found in the workspace
-        self._workspace_folder_uri_to_folder_info = {}
-        self._additional_pythonpath_folder_to_folder_info = {}
+        self._workspace_folder_uri_to_folder_info: Dict[str, _FolderInfo] = {}
+        self._additional_pythonpath_folder_to_folder_info: Dict[str, _FolderInfo] = {}
 
         # Spec info found in the pythonpath
-        pythonpath_folder_to_folder_info = {}
+        pythonpath_folder_to_folder_info: Dict[str, _FolderInfo] = {}
         for path in sys.path:
             if path and os.path.isdir(path):
                 pythonpath_folder_to_folder_info[path] = _FolderInfo(
                     path, recursive=False
                 )
-        self._pythonpath_folder_to_folder_info = pythonpath_folder_to_folder_info
+        self._pythonpath_folder_to_folder_info: Dict[
+            str, _FolderInfo
+        ] = pythonpath_folder_to_folder_info
 
         # Spec info found in internal dirs (autogenerated)
-        self._internal_folder_to_folder_info = {
+        self._internal_folder_to_folder_info: Dict[str, _FolderInfo] = {
             self._user_libspec_dir: _FolderInfo(
                 self._user_libspec_dir, recursive=False
             ),
@@ -495,6 +482,10 @@ class LibspecManager(object):
         self._synchronize()
         log.debug("Finished initializing LibspecManager.")
 
+    @property
+    def fs_observer(self) -> IFSObserver:
+        return self._fs_observer
+
     def _check_in_main_thread(self):
         curr_thread = threading.current_thread()
         if self._main_thread is not curr_thread:
@@ -565,7 +556,7 @@ class LibspecManager(object):
                 uris.to_fs_path(folder_uri), recursive=True
             )
             self._workspace_folder_uri_to_folder_info = cp
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
         else:
             log.debug("Workspace folder already added: %s", folder_uri)
@@ -588,7 +579,7 @@ class LibspecManager(object):
             cp = self._additional_pythonpath_folder_to_folder_info.copy()
             folder_info = cp[folder_path] = _FolderInfo(folder_path, recursive=True)
             self._additional_pythonpath_folder_to_folder_info = cp
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
         else:
             log.debug("Additional pythonpath folder already added: %s", folder_path)
@@ -663,22 +654,22 @@ class LibspecManager(object):
 
     def synchronize_workspace_folders(self):
         for folder_info in self._workspace_folder_uri_to_folder_info.values():
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
 
     def synchronize_pythonpath_folders(self):
         for folder_info in self._pythonpath_folder_to_folder_info.values():
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
 
     def synchronize_additional_pythonpath_folders(self):
         for folder_info in self._additional_pythonpath_folder_to_folder_info.values():
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
 
     def synchronize_internal_libspec_folders(self):
         for folder_info in self._internal_folder_to_folder_info.values():
-            folder_info.start_watch(self._observer, self._file_changes_notifier)
+            folder_info.start_watch(self._fs_observer, self._file_changes_notifier)
             folder_info.synchronize()
 
     def _synchronize(self):
@@ -694,7 +685,7 @@ class LibspecManager(object):
         self.synchronize_additional_pythonpath_folders()
         self.synchronize_internal_libspec_folders()
 
-    def iter_lib_info(self):
+    def iter_lib_info(self, builtin=False):
         """
         :rtype: generator(_LibInfo)
         """
@@ -713,9 +704,14 @@ class LibspecManager(object):
             if info.libspec_canonical_filename_to_info:
                 iter_in.append((info.libspec_canonical_filename_to_info, False))
 
-        for (_uri, info) in self._internal_folder_to_folder_info.items():
+        if builtin:
+            info = self._internal_folder_to_folder_info[self._builtins_libspec_dir]
             if info.libspec_canonical_filename_to_info:
                 iter_in.append((info.libspec_canonical_filename_to_info, True))
+        else:
+            for (_uri, info) in self._internal_folder_to_folder_info.items():
+                if info.libspec_canonical_filename_to_info:
+                    iter_in.append((info.libspec_canonical_filename_to_info, True))
 
         for canonical_filename_to_info, can_regenerate in iter_in:
             for canonical_spec_filename, info in list(
@@ -944,7 +940,6 @@ class LibspecManager(object):
                 log.debug("Took: %.2fs to generate info for: %s" % (delta, libname))
 
     def dispose(self):
-        self._observer.dispose()
         self._file_changes_notifier.dispose()
 
     def _do_create_libspec_on_get(self, libname, target_file: Optional[str]):
@@ -1000,7 +995,11 @@ class LibspecManager(object):
         return target_file
 
     def get_library_info(
-        self, libname: str, create: bool = True, current_doc_uri: Optional[str] = None
+        self,
+        libname: str,
+        create: bool = True,
+        current_doc_uri: Optional[str] = None,
+        builtin: bool = False,
     ) -> Optional[ILibraryDoc]:
         """
         :param libname:
@@ -1011,15 +1010,19 @@ class LibspecManager(object):
         """
 
         libname_lower = libname.lower()
-        target_file = self._get_library_target_filename(libname, current_doc_uri)
+        target_file: str = ""
+        normalized_target_file: str = ""
 
-        if target_file:
-            normalized_target_file = os.path.normcase(os.path.normpath(target_file))
-        else:
-            normalized_target_file = ""
+        if not builtin:
+            found_target_filename = self._get_library_target_filename(
+                libname, current_doc_uri
+            )
+            if found_target_filename:
+                target_file = found_target_filename
+                normalized_target_file = os.path.normcase(os.path.normpath(target_file))
 
         lib_info: _LibInfo
-        for lib_info in self.iter_lib_info():
+        for lib_info in self.iter_lib_info(builtin=builtin):
             library_doc = lib_info.library_doc
 
             # If it maps to a file in the filesystem, that's what we need to match,
@@ -1060,7 +1063,10 @@ class LibspecManager(object):
                         # Note: get even if it if was not created (we may match
                         # a lower priority library).
                         return self.get_library_info(
-                            libname, create=False, current_doc_uri=current_doc_uri
+                            libname,
+                            create=False,
+                            current_doc_uri=current_doc_uri,
+                            builtin=builtin,
                         )
                     else:
                         # Not in sync and it should not be created, just skip it.
@@ -1071,7 +1077,10 @@ class LibspecManager(object):
         if create:
             if self._do_create_libspec_on_get(libname, target_file):
                 return self.get_library_info(
-                    libname, create=False, current_doc_uri=current_doc_uri
+                    libname,
+                    create=False,
+                    current_doc_uri=current_doc_uri,
+                    builtin=builtin,
                 )
 
         log.debug("Unable to find library named: %s", libname)
