@@ -1,10 +1,10 @@
-from robotframework_ls.impl.completion_context import CompletionContext
 from typing import List, Tuple
+import itertools
 from robocorp_ls_core.protocols import IDocument
+from robotframework_ls.impl.protocols import ICompletionContext
 
 
 TOKEN_TYPES = [
-    "parameter",
     "variable",
     "comment",
     # Custom added
@@ -17,6 +17,8 @@ TOKEN_TYPES = [
     "settingOperator",  # keyword.operator.setting.robot
     "control",
     "testCaseName",
+    "parameterName",
+    "argumentValue",
 ]
 
 TOKEN_MODIFIERS = [
@@ -67,7 +69,7 @@ RF_TOKEN_TYPE_TO_TOKEN_TYPE_INDEX = {
     KEYWORD_NAME: TOKEN_TYPE_TO_INDEX["keywordNameDefinition"],
     TESTCASE_NAME: TOKEN_TYPE_TO_INDEX["testCaseName"],
     KEYWORD: TOKEN_TYPE_TO_INDEX["keywordNameCall"],
-    ARGUMENT: TOKEN_TYPE_TO_INDEX["parameter"],
+    ARGUMENT: TOKEN_TYPE_TO_INDEX["argumentValue"],
     VARIABLE: TOKEN_TYPE_TO_INDEX["variable"],
 }
 
@@ -88,6 +90,7 @@ for key, val in list(RF_TOKEN_TYPE_TO_TOKEN_TYPE_INDEX.items()):
 VARIABLE_INDEX = TOKEN_TYPE_TO_INDEX["variable"]
 VARIABLE_OPERATOR_INDEX = TOKEN_TYPE_TO_INDEX["variableOperator"]
 SETTING_INDEX = TOKEN_TYPE_TO_INDEX["setting"]
+PARAMETER_NAME_INDEX = TOKEN_TYPE_TO_INDEX["parameterName"]
 
 
 class _DummyToken(object):
@@ -101,14 +104,79 @@ def semantic_tokens_range(context, range):
     return []
 
 
-def tokenize_variables(initial_token):
+def tokenize_variables(node, initial_token):
     try:
+
         iter_in = initial_token.tokenize_variables()
     except:
         token_type_index = RF_TOKEN_TYPE_TO_TOKEN_TYPE_INDEX.get(initial_token.type)
         if token_type_index is not None:
             yield initial_token, token_type_index
     else:
+        if (
+            initial_token.type == ARGUMENT
+            and node.__class__.__name__ != "Documentation"
+        ):
+
+            first_token = next(iter_in)
+            equals_pos = first_token.value.find("=")
+            if equals_pos != -1:
+                # Found an equals... let's check if it's not a 'catenate', which
+                # doesn't really accept parameters and just concatenates all...
+                value = node.get_value(initial_token.KEYWORD)
+                if value and value.strip().lower() == "catenate":
+                    equals_pos = -1
+
+                    # Note: the best way to actually do this would be finding the
+                    # reference to the keyword and then validating whether the
+                    # keyword arguments match the expected name.
+                    #
+                    # For instance, a keyword call such as:
+                    # Some Call     some arg = 22
+                    #
+                    # Should color `some arg =` differently only if the argument
+                    # of `Some Call` is `some arg`, otherwise it should not color
+                    # the argument as `same arg = 22` will be passed as a string
+                    # to the positional argument 0 and not really a keyword parameter
+                    # where `same arg` is set with value 22.
+                    #
+                    # Now, this requires a bit more tinkering with keyword caches
+                    # and possibly semantic highlighting deltas to make sure the
+                    # performance isn't negatively impacted by it.
+
+            if equals_pos != -1:
+                tok = _DummyToken()
+                tok.type = "parameterName"
+                tok.value = first_token.value[:equals_pos]
+                tok.lineno = first_token.lineno
+                tok.col_offset = first_token.col_offset
+                prev_col_offset_end = tok.end_col_offset = first_token.col_offset + len(
+                    tok.value
+                )
+                yield tok, PARAMETER_NAME_INDEX
+
+                tok = _DummyToken()
+                tok.type = "variableOperator"
+                tok.value = "="
+                tok.lineno = first_token.lineno
+                tok.col_offset = prev_col_offset_end
+                prev_col_offset_end = tok.end_col_offset = prev_col_offset_end + 1
+                yield tok, VARIABLE_OPERATOR_INDEX
+
+                # Add the remainder back.
+                first_token_remainder = _DummyToken()
+                first_token_remainder.type = first_token.type
+                first_token_remainder.value = first_token.value[equals_pos + 1 :]
+                first_token_remainder.lineno = first_token.lineno
+                first_token_remainder.col_offset = prev_col_offset_end
+                prev_col_offset_end = (
+                    first_token_remainder.end_col_offset
+                ) = prev_col_offset_end + len(first_token_remainder.value)
+
+                iter_in = itertools.chain(iter([first_token_remainder]), iter_in)
+            else:
+                iter_in = itertools.chain(iter((first_token,)), iter_in)
+
         for token in iter_in:
 
             token_type_index = RF_TOKEN_TYPE_TO_TOKEN_TYPE_INDEX.get(token.type)
@@ -185,7 +253,7 @@ def tokenize_variables(initial_token):
                     yield token, token_type_index
 
 
-def semantic_tokens_full(context: CompletionContext):
+def semantic_tokens_full(context: ICompletionContext):
     from robotframework_ls.impl import ast_utils
 
     try:
@@ -203,7 +271,7 @@ def semantic_tokens_full(context: CompletionContext):
         tokens = getattr(node, "tokens", None)
         if tokens:
             for token in tokens:
-                for token_part, token_type_index in tokenize_variables(token):
+                for token_part, token_type_index in tokenize_variables(node, token):
                     lineno = token_part.lineno - 1
                     if lineno < 0:
                         lineno = 0
