@@ -9,7 +9,7 @@ import sys
 import weakref
 import os
 from robocorp_ls_core.robotframework_log import get_logger
-from typing import Any, Dict, Optional, Tuple
+from typing import Any, Dict, Optional, Tuple, List, Iterable
 from robotframework_ls.ep_resolve_interpreter import (
     EPResolveInterpreter,
     IInterpreterInfo,
@@ -197,6 +197,7 @@ class _ServerApi(object):
         return env
 
     def get_robotframework_api_client(self) -> Optional[IRobotFrameworkApiClient]:
+
         self._check_in_main_thread()
         workspace = self.workspace
         assert (
@@ -223,6 +224,9 @@ class _ServerApi(object):
                     JsonRpcStreamWriter,
                     JsonRpcStreamReader,
                 )
+                from robotframework_ls.robotframework_ls_impl import (
+                    RobotFrameworkLanguageServer,
+                )
 
                 args = []
                 if Setup.options.verbose:
@@ -249,6 +253,16 @@ class _ServerApi(object):
                 self._used_python_executable = python_exe
                 self._used_environ = environ
 
+                robot_framework_language_server: RobotFrameworkLanguageServer = self.robot_framework_language_server
+                remote_fs_observer_port = (
+                    robot_framework_language_server.get_remote_fs_observer_port()
+                )
+                if not remote_fs_observer_port:
+                    raise RuntimeError(
+                        f"Expected the port to hear the Remote filesystem observer to be available. Found: {remote_fs_observer_port}"
+                    )
+
+                args.append(f"--remote-fs-observer-port={remote_fs_observer_port}")
                 server_process = start_server_process(
                     args=args, python_exe=python_exe, env=environ
                 )
@@ -413,7 +427,7 @@ class _RegularLintAndOthersApi(object):
     document formatting, code folding, semantic tokens and workspace symbols.
     """
 
-    def __init__(self, api, lint_api, others_api):
+    def __init__(self, api: _ServerApi, lint_api: _ServerApi, others_api: _ServerApi):
         self.api = api
         self.lint_api = lint_api
         self.others_api = others_api
@@ -463,7 +477,7 @@ class ServerManager(object):
                 f"This may only be called at the thread: {self._main_thread}. Current thread: {curr_thread}"
             )
 
-    def _iter_all_apis(self):
+    def _iter_all_apis(self) -> Iterable[_ServerApi]:
         self._check_in_main_thread()
         for apis in self._id_to_apis.values():
             for api in apis:
@@ -558,6 +572,9 @@ class ServerManager(object):
         for api in self._iter_all_apis():
             api.exit()
 
+    def collect_apis(self) -> List[_ServerApi]:
+        return list(self._iter_all_apis())
+
     # Private APIs
 
     def _get_others_api(self, doc_uri: str) -> _ServerApi:
