@@ -12,7 +12,10 @@ from robocop.utils.utils import (
     DISABLED_IN_4,
     ENABLED_IN_4,
     keyword_col,
-    issues_to_lsp_diagnostic
+    issues_to_lsp_diagnostic,
+    AssignmentTypeDetector,
+    parse_assignment_sign_type,
+    token_col
 )
 
 
@@ -28,5 +31,8 @@ __all__ = [
     'DISABLED_IN_4',
     'ENABLED_IN_4',
     'keyword_col',
-    'issues_to_lsp_diagnostic'
+    'issues_to_lsp_diagnostic',
+    'AssignmentTypeDetector',
+    'parse_assignment_sign_type',
+    'token_col'
 ]
