@@ -344,6 +344,9 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
             If None (default), the contents will be loaded from the disk.
         """
 
+    def close_doc(self, uri: str):
+        pass
+
     def change_doc(self, uri: str, version: int, text: str):
         pass
 
@@ -434,7 +437,10 @@ class IConfig(Protocol):
     def get_full_settings(self) -> dict:
         pass
 
-    def set_override_settings(self, override_settings: dict):
+    def set_override_settings(self, override_settings: dict) -> None:
+        pass
+
+    def set_workspace_dir(self, workspace: str) -> None:
         pass
 
 
@@ -556,6 +562,9 @@ class IDocument(Protocol):
     def get_line(self, line: int) -> str:
         pass
 
+    def get_range(self, line: int, col: int, endline: int, endcol: int) -> str:
+        pass
+
     def get_last_line(self) -> str:
         pass
 
@@ -633,3 +642,35 @@ class IMonitor(Protocol):
         """
         raises JsonRpcRequestCancelled if cancelled.
         """
+
+
+class ActionResult(Generic[T]):
+
+    success: bool
+    message: Optional[
+        str
+    ]  # if success == False, this can be some message to show to the user
+    result: Optional[T]
+
+    def __init__(
+        self, success: bool, message: Optional[str] = None, result: Optional[T] = None
+    ):
+        self.success = success
+        self.message = message
+        self.result = result
+
+    def as_dict(self):
+        return {"success": self.success, "message": self.message, "result": self.result}
+
+    def __str__(self):
+        return f"ActionResult(success={self.success!r}, message={self.message!r}, result={self.result!r})"
+
+    __repr__ = __str__
+
+
+class ActionResultDict(TypedDict):
+    success: bool
+    message: Optional[
+        str
+    ]  # if success == False, this can be some message to show to the user
+    result: Any
